@@ -1,10 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
-    2012, 2013, 2014 Antonio Diaz Diaz.
+    Copyright (C) 2003-2015 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -103,6 +102,8 @@ int Features::test_235Esz( const Charset & charset ) const
           ( lp[lrow2 + 1 - b.top()] < lcol2 - b.left() ||
             lp[urow3 - 1 - b.top()] < ucol3 - b.left() ) )
         {
+        for( int i = lp.pos( 40 ); i <= lp.pos( 70 ); ++i )
+          if( 5 * lp[i] < b.width() && 2 * lp[i+1] > b.width() ) return '5';
         int c = 0, hdiff;
         if( !b.top_hook( &hdiff ) || 5 * hdiff >= 4 * b.height() ) ++c;
         if( 2 * lp[lrow2 - b.top()] < lcol2 - b.left() ) ++c;
@@ -137,15 +138,17 @@ int Features::test_235Esz( const Charset & charset ) const
               else return UCS::SCCEDI; }
           return 'F';
           }
-        else if( lrow1 < urow2 && lrow2 < urow3 ) return 'E';
+        else if( lrow1 < urow2 && urow2 + 2 < lrow2 && lrow2 < urow3 &&
+                 urow2 <= b.vcenter() && lrow2 >= b.vcenter() ) return 'E';
         }
       }
     else if( b.escape_left( urow2, ucol2 ) )
       {
       if( !tbopen && ( 2 * lp[lp.pos(50)] ) + 2 >= b.width() &&
-          ( tp.isconvex() || ( tp.ispit() && !bp.ispit() ) ) )
+          ( tp.isconvex() || ( (tp.ispit() || tp.isrtip()) && !bp.ispit() ) ) )
         return '2';
-      if( b.height() <= 2 * wp.max() && bp[bp.pos(75)] <= b.height() / 10 )
+      if( 2 * b.height() <= 5 * wp.max() && bp[bp.pos(75)] <= b.height() / 10 &&
+          Ocrad::similar( wp.max( 0, wp.pos(30) ), wp.max( wp.pos(70) ), 20 ) )
         return 'z';
       }
     }
@@ -153,27 +156,26 @@ int Features::test_235Esz( const Charset & charset ) const
   }
 
 
-int Features::test_CEFIJLlT( const Charset & charset ) const
+int Features::test_EFIJLlT( const Charset & charset, const Rectangle & charbox ) const
   {
   if( tp.minima( b.height() / 4 ) != 1 || bp.minima( b.height() / 4 ) != 1 )
     return 0;
 
   const int noise = ( std::min( b.height(), b.width() ) / 30 ) + 1;
   {
-  int col;
-  if( 2 * ( lp[lp.pos(50)] + noise ) >= b.width() ) col = b.hpos( 25 );
-  else col = b.hpos( 75 );
+  const bool maybe_j = ( 2 * ( lp[lp.pos(50)] + noise ) >= b.width() );
+  const int col = b.hpos( maybe_j ? 25 : 75 );
   int row = b.seek_top( b.vcenter(), col );
   if( row <= b.top() || ( row < b.vpos( 25 ) && b.escape_top( row, col ) ) )
     {
     int hdiff;
     if( b.bottom_hook( &hdiff ) )
       {
-      if( hdiff > b.height() / 2 && rp.increasing( rp.pos( 80 ), 1 ) &&
-          !rp.decreasing() ) return 'J';
-      if( -hdiff > b.height() / 2 )
+      if( maybe_j && hdiff > b.height() / 2 &&
+          rp.increasing( rp.pos( 80 ), 1 ) && !rp.decreasing() ) return 'J';
+      if( !maybe_j && -hdiff > b.height() / 2 )
         {
-        if( 5 * lp[lp.pos(80)] >= 2 * b.width() ) return 'v';
+        if( 5 * lp[lp.pos(80)] >= 2 * b.width() ) return 'v';	// broken 'v'
         if( col > b.hcenter() ) return 'L';
         }
       }
@@ -183,16 +185,19 @@ int Features::test_CEFIJLlT( const Charset & charset ) const
   const int vnoise = ( b.height() / 30 ) + 1;
   const int topmax = b.top() + vnoise;
   const int botmin = b.bottom() - vnoise;
-  if( vbars() == 1 && vbar(0).width() >= 2 && 2 * vbar(0).width() < b.width() )
+  if( vbars() == 1 && vbar(0).width() >= 2 && 2 * vbar(0).width() <= b.width() )
     {
     if( std::abs( vbar(0).hcenter() - b.hcenter() ) <= noise &&
         std::abs( (vbar(0).left() - b.left()) - (b.right() - vbar(0).right()) ) <= 2 * noise )
       {
       if( hbars() == 1 && 4 * hbar(0).height() <= b.height() )
         {
-        if( hbar(0).top() <= topmax || hbar(0).bottom() < b.vpos( 15 ) )
+        if( ( hbar(0).top() <= topmax || hbar(0).bottom() < b.vpos( 15 ) ) &&
+            hbar(0).width() >= wp[wp.pos(75)] + wp[wp.pos(80)] &&
+            4 * lp[lp.pos(50)] >= b.width() )
           return 'T';
-        if( std::abs( hbar(0).vcenter() - b.vcenter() ) <= 1 &&
+        if( std::abs( hbar(0).vcenter() - b.vcenter() ) <= vnoise &&
+            hbar(0).width() >= b.width() &&
             Ocrad::similar( b.height(), b.width(), 50 ) ) return '+';
         }
       if( hbars() == 2 &&
@@ -203,10 +208,9 @@ int Features::test_CEFIJLlT( const Charset & charset ) const
       }
     }
 
-  if( vbars() == 1 && vbar(0).width() >= 2 &&
-      2 * vbar(0).width() <= b.width() )
+  if( vbars() == 1 && vbar(0).width() >= 2 )
     {
-    if( vbar(0).right() <= b.hcenter() )
+    if( 2 * vbar(0).width() <= b.width() && vbar(0).right() <= b.hcenter() )
       {
       if( ( hbars() == 2 || hbars() == 3 ) && hbar(0).top() <= topmax &&
           hbar(0).width() + 1 >= hbar(1).width() &&
@@ -243,14 +247,21 @@ int Features::test_CEFIJLlT( const Charset & charset ) const
         }
       }
 
-    if( vbar(0).left() > b.hcenter() &&
-        hbars() == 1 && hbar(0).top() <= topmax &&
-        hbar(0).width() + 1 >= b.width() )
+    if( 3 * vbar(0).width() < 2 * b.width() && vbar(0).left() > b.hpos( 33 ) &&
+        hbars() == 1 )
       {
-      if( charset.enabled( Charset::iso_8859_15 ) ||
-          charset.enabled( Charset::iso_8859_9 ) )
-        if( b.width() > b.height() ) return UCS::NOT;
-      return 0;
+      if( vbar(0).right() >= b.hpos( 90 ) && hbar(0).bottom() >= botmin &&
+          hbar(0).left() == b.left() &&
+          b.bottom() > charbox.vpos( 90 ) &&
+          b.escape_top( b.vcenter(), b.hpos( 25 ) ) )
+        { if( b.height() > b.width() ) return 'J'; else return 0; }
+      if( hbar(0).top() <= topmax && hbar(0).width() + 1 >= b.width() &&
+          b.width() > b.height() )
+        {
+        if( charset.enabled( Charset::iso_8859_15 ) ||
+            charset.enabled( Charset::iso_8859_9 ) ) return UCS::NOT;
+        return 0;
+        }
       }
     }
 
@@ -289,22 +300,6 @@ int Features::test_CEFIJLlT( const Charset & charset ) const
               if( b.seek_bottom( b.vpos( 75 ), i ) < b.bottom() &&
                   bp[i-b.left()] <= noise ) return 'l';
       }
-    if( vbar(0).left() <= b.left() + 1 && b.height() > 2 * b.width() &&
-        rp.istip() )
-      {
-      if( 2 * rp[rp.pos(50)] > b.width() )
-        {
-        int row = b.seek_top( b.vcenter(), b.hcenter() );
-        int col = b.seek_right( row, b.hcenter() );
-        if( col < b.right() )
-          {
-          row = b.seek_bottom( b.vcenter(), b.hcenter() );
-          col = b.seek_right( row, b.hcenter() );
-          if( col < b.right() ) return 'C';
-          }
-        }
-      return '[';
-      }
     if( vbar(0).right() >= b.right() - 1 )
       {
       if( lp.istip() && b.height() > 2 * b.width() )
@@ -320,7 +315,8 @@ int Features::test_CEFIJLlT( const Charset & charset ) const
           if( i > lp.pos( 10 ) && i < lp.pos( 40 ) ) return '1'; }
       }
     }
-  if( hbars() == 1 && std::abs( hbar(0).vcenter() - b.vcenter() ) <= 1 &&
+  if( hbars() == 1 && hbar(0).width() >= b.width() &&
+      std::abs( hbar(0).vcenter() - b.vcenter() ) <= vnoise &&
       Ocrad::similar( b.height(), b.width(), 50 ) &&
       tp.isupit() && bp.isupit() )
     return '+';
@@ -335,13 +331,11 @@ int Features::test_c() const
     int urow = b.seek_top( b.vcenter(), b.hcenter() );
     int lrow = b.seek_bottom( b.vcenter(), b.hcenter() );
 
-    if( b.height() > 2 * b.width() &&
-        ( 3 * wp.max() <= 2 * b.width() ||
-        ( 2 * rp[urow-b.top()] >= b.width() && 2 * rp[lrow-b.top()] >= b.width() ) ) )
+    if( b.height() > 2 * b.width() && 3 * wp.max() <= 2 * b.width() )
       { if( lp.isconvex() ) return '('; else return 0; }
 
     if( urow > b.top() && lrow < b.bottom() && rp.isctip() &&
-        ( bp.ispit() || tp.ispit() || ( bp.islpit() && tp.islpit() ) ) &&
+        ( bp.ispit() || tp.ispit() || ( bp.isltip() && tp.isltip() ) ) &&
         b.escape_right( b.vcenter(), b.hcenter() ) )
       return 'c';
     }
@@ -375,7 +369,7 @@ int Features::test_frst( const Rectangle & charbox ) const
       {
       if( -2 * b_hdiff > b.height() )
         {
-        if( b.height() >= 3 * wp.max() &&
+        if( b.height() >= 3 * wp.max() && !lp.ispit() &&
             ( hbars() == 0 || hbar(0).bottom() < b.vpos( 20 ) ) ) return 'l';
         if( 2 * wp[wp.pos(6)] < b.width() && hbars() >= 1 && hbars() <= 2 &&
             hbar(0).top() >= b.vpos( 15 ) && hbar(0).bottom() < b.vcenter() &&
@@ -425,7 +419,8 @@ int Features::test_frst( const Rectangle & charbox ) const
                 4 * hbar(0).height() <= b.height() &&
                 4 * lp[lp.pos(50)] >= b.width() )
               return 'T';
-            return 'r';
+            if( 3 * rp[rp.pos(50)] > b.width() ) return 'r';
+            return 0;
             }
           }
         }
@@ -453,7 +448,7 @@ int Features::test_frst( const Rectangle & charbox ) const
               tp.minima( noise ) == 2 ||
               ( rp.minima() == 1 && ( b.height() < charbox.height() || tp.iminimum() > tp.pos( 50 ) ) ) )
             { if( b.height() <= 3 * wp.max() ) return 'r'; else return 0; }
-          else if( 3 * b.height() >= 5 * b.width() ) return 'f';
+          else if( 3 * b.height() >= 5 * b.width() && !rp.istip() ) return 'f';
           }
         else
           {
@@ -479,7 +474,7 @@ int Features::test_G() const
   if( lp.isconvex() || lp.ispit() )
     {
     int col = 0, row = 0;
-    for( int i = rp.pos( 30 ); i <= rp.pos( 60 ); ++i )
+    for( int i = rp.pos( 60 ); i >= rp.pos( 30 ); --i )
       if( rp[i] > col ) { col = rp[i]; row = i; }
     if( col == 0 ) return 0;
     row += b.top(); col = b.right() - col + 1;
@@ -496,9 +491,10 @@ int Features::test_G() const
         if( b.seek_right( lrow, b.hcenter() ) >= b.right() ) break;
       for( urow = lrow - 1 ; urow > b.top(); --urow )
         if( b.seek_right( urow, b.hcenter() ) < b.right() ) break;
-      lrow += noise; urow -= noise;
+      lrow += noise;
       if( lrow < row && urow > b.top() )
         {
+        urow -= std::min( noise, ( urow - b.top() ) / 2 );
         int uwidth = b.seek_left( urow, b.right() ) - b.seek_right( urow, b.hcenter() );
         int lwidth = b.seek_left( lrow, b.right() ) - b.seek_right( lrow, b.hcenter() );
         if( lrow - noise <= b.vcenter() || lwidth > uwidth + noise )
@@ -523,9 +519,14 @@ int Features::test_HKMNUuvwYy( const Rectangle & charbox ) const
         ( m5 == 1 ||
           ( m5 == 2 && Ocrad::similar( bp.iminimum(), bp.pos( 50 ), 10 ) ) ) )
       {
+      const int stem = std::min( tp.range() + ( b.height() / 10 ), wp.pos(90) );
+      const bool maybe_Y = ( 5 * tp.range() <= 3 * b.height() ||
+        ( stem <= wp.pos(75) && 5 * wp[stem] <= b.width() ) );
       const int lg = lp.min( lp.pos( 90 ) );
       if( lg > 1 && bp.isvpit() && tp.minima( b.height() / 2 ) == 2 &&
-          lp[lp.pos(75)] <= lg ) return 'v';
+          lp[lp.pos(75)] <= lg &&
+          ( !maybe_Y || 3 * wp[stem] > b.width() || wp[stem] > wp[wp.pos(90)] + 1 ) )
+        return 'v';
       int hdiff;
       if( b.bottom_hook( &hdiff ) )
         {
@@ -545,7 +546,7 @@ int Features::test_HKMNUuvwYy( const Rectangle & charbox ) const
       const int rg2 = rp.max( rp.pos( 70 ), rp.pos( 90 ) );
       const int lc = ( lg + ( 2 * ( lp.limit() - rg ) ) ) / 3;
       const int lc2 = ( lg2 + lp.limit() - rg2 ) / 2;
-      if( bp.ispit() && 7 * tp.range() < 4 * b.height() )
+      if( bp.ispit() && maybe_Y )
         {
         int row2 = b.top();
         while( row2 < b.bottom() && segments_in_row( row2 ) != 2 ) ++row2;
@@ -562,8 +563,9 @@ int Features::test_HKMNUuvwYy( const Rectangle & charbox ) const
             return 'Y';
           }
         }
-      if( b.escape_top( b.vpos( 60 ), b.hcenter() ) && !lp.istip() )
-        return 'u';
+      if( b.escape_top( b.vpos( 60 ), b.hcenter() ) && !lp.istip() &&
+          ( 4 * b.height() >= 3 * b.width() ||
+            segments_in_col( b.hpos( 75 ) ) <= 2 ) ) return 'u';
       if( lg < rg + 1 && !lp.increasing( lp.pos( 50 ) ) &&
           ( 2 * lg < rg || b.vpos( 90 ) >= charbox.bottom() ) &&
           ( tp.minima( b.height()/2 ) == 1 || lp.imaximum() > b.height()/2 ) )
@@ -590,10 +592,10 @@ int Features::test_HKMNUuvwYy( const Rectangle & charbox ) const
         if( hbars() == 1 && 5 * ( hbar(0).height() - 1 ) < b.height() &&
             hbar(0).top() >= b.vpos( 30 ) && hbar(0).bottom() <= b.vpos( 60 ) &&
             10 * hbar(0).width() > 9 * wp[hbar(0).vcenter()-b.top()] &&
-            Ocrad::similar( col_segment( hbar(0).vcenter(), hbar(0).hcenter() ).size(),
+            Ocrad::similar( v_segment( hbar(0).vcenter(), hbar(0).hcenter() ).size(),
                             hbar(0).height(), 30, 2 ) )
           {
-          if( 9 * hbar(0).width() < 10 * wp[wp.pos(50)] ) return 'H';
+          if( 9 * hbar(0).width() <= 10 * wp[wp.pos(50)] ) return 'H';
           return 0;
           }
         if( segments_in_row( b.vpos( 60 ) ) == 4 ||
@@ -640,7 +642,7 @@ int Features::test_hknwx( const Rectangle & charbox ) const
   const int m8 = tp.minima( b.height() / 8 );
 
   if( m8 == 2 && bp.minima( b.height() / 2 ) == 1 &&
-      ( ( lp.istip() && rp.istip() ) ||
+      ( ( lp.isctip() && rp.isctip() ) ||
         ( lp.isconcave() && rp.isconcave() ) ) ) return 'x';
 
   if( b.width() >= b.height() && tp.ispit() &&
@@ -660,6 +662,8 @@ int Features::test_hknwx( const Rectangle & charbox ) const
   if( urow > b.vpos( 20 ) || 3 * tp[tp.pos(60)] > b.height() )
     {
     const int m5 = tp.minima( b.height() / 5 );
+    if( m5 == 3 && segments_in_row( b.vcenter() ) == 2 &&
+        segments_in_row( b.vpos( 80 ) ) == 3 ) return 0;	// merged 'IX'
     if( ( m5 == 2 || m5 == 3 ) && tp.minima() >= 2 &&
         rp[rp.pos(25)] <= b.width() / 4 &&
         ( !lp.istpit() || rp.minima() == 1 ) ) return 'w';
@@ -674,7 +678,7 @@ int Features::test_hknwx( const Rectangle & charbox ) const
   if( urow <= b.vpos( 20 ) && tp.minima( b.height() / 4 ) == 1 &&
       Ocrad::similar( b.height(), b.width(), 40 ) &&
       ( 8 * ( rp[rp.pos(50)] - 1 ) <= b.width() ||
-        tp[tp.pos(100)] > b.height() / 2 ) )
+        tp[tp.pos(99)] > b.height() / 2 ) )
     return 'n';
   return 0;
   }
@@ -754,10 +758,36 @@ int Features::test_easy( const Rectangle & charbox ) const
 
 
 // Recognizes single line, non-rectangular characters without holes.
-// '/<>\^`
+// '/<>C[\^`c
 //
 int Features::test_line( const Rectangle & charbox ) const
   {
+  const int vnoise = ( b.height() / 30 ) + 1;
+  const int topmax = b.top() + vnoise;
+  const int botmin = b.bottom() - vnoise;
+  const bool vbar_left = ( vbars() == 1 && vbar(0).width() >= 2 &&
+                           vbar(0).left() <= b.hpos( 10 ) + 1 );
+  if( tp.minima() == 1 && bp.minima() == 1 && rp.istip() )
+    {
+    if( vbar_left && b.height() > 2 * b.width() &&
+        2 * rp[rp.pos(50)] > b.width() )
+      {
+      int row = b.seek_top( b.vcenter(), b.hcenter() );
+      int col = b.seek_right( row, b.hcenter() );
+      if( col < b.right() )
+        {
+        row = b.seek_bottom( b.vcenter(), b.hcenter() );
+        col = b.seek_right( row, b.hcenter() );
+        if( col < b.right() ) return 'C';
+        }
+      }
+    if( hbars() == 2 &&
+        hbar(0).top() <= topmax && 4 * hbar(0).height() <= b.height() &&
+        hbar(1).bottom() >= botmin && 4 * hbar(1).height() <= b.height() )
+      { if( vbar_left && b.height() > 2 * b.width() ) return '[';
+        if( vbar_left || lp.ispit() ) return 'c'; }
+    }
+
   int slope1, slope2;
 
   if( tp.minima() != 1 ) return 0;
