@@ -1,10 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
-    2012, 2013, 2014 Antonio Diaz Diaz.
+    Copyright (C) 2003-2015 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -40,6 +39,7 @@
 #include "common.h"
 #include "rational.h"
 #include "rectangle.h"
+#include "user_filter.h"
 #include "page_image.h"
 #include "textpage.h"
 
@@ -48,7 +48,7 @@ namespace {
 
 const char * const Program_name = "GNU Ocrad";
 const char * const program_name = "ocrad";
-const char * const program_year = "2014";
+const char * const program_year = "2015";
 const char * invocation_name = 0;
 
 struct Input_control
@@ -75,8 +75,9 @@ void show_error( const char * const msg, const int errcode = 0,
     if( msg && msg[0] )
       {
       std::fprintf( stderr, "%s: %s", program_name, msg );
-      if( errcode > 0 ) std::fprintf( stderr, ": %s", std::strerror( errcode ) );
-      std::fprintf( stderr, "\n" );
+      if( errcode > 0 )
+        std::fprintf( stderr, ": %s.", std::strerror( errcode ) );
+      std::fputs( "\n", stderr );
       }
     if( help )
       std::fprintf( stderr, "Try '%s --help' for more information.\n",
@@ -125,32 +126,38 @@ void show_help()
                "UTF-8 formats. The pbm, pgm and ppm formats are collectively known as pnm.\n"
                "\nOcrad includes a layout analyser able to separate the columns or blocks\n"
                "of text normally found on printed pages.\n"
+               "\nFor best results the characters should be at least 20 pixels high. If\n"
+               "they are smaller, try the --scale option. Scanning the image at 300 dpi\n"
+               "usually produces a character size good enough for ocrad.\n"
+               "Merged, very bold or very light (broken) characters are normally not\n"
+               "recognized correctly. Try to avoid them.\n"
                "\nUsage: %s [options] [files]\n", invocation_name );
   std::printf( "\nOptions:\n"
-               "  -h, --help               display this help and exit\n"
-               "  -V, --version            output version information and exit\n"
-               "  -a, --append             append text to output file\n"
-               "  -c, --charset=<name>     try '--charset=help' for a list of names\n"
-               "  -e, --filter=<name>      try '--filter=help' for a list of names\n"
-               "  -f, --force              force overwrite of output file\n"
-               "  -F, --format=<fmt>       output format (byte, utf8)\n"
-               "  -i, --invert             invert image levels (white on black)\n"
-               "  -l, --layout             perform layout analysis\n"
-               "  -o, --output=<file>      place the output into <file>\n"
-               "  -q, --quiet              suppress all messages\n"
-               "  -s, --scale=[-]<n>       scale input image by [1/]<n>\n"
-               "  -t, --transform=<name>   try '--transform=help' for a list of names\n"
-               "  -T, --threshold=<n%%>     threshold for binarization (0-100%%)\n"
-               "  -u, --cut=<l,t,w,h>      cut input image by given rectangle\n"
-               "  -v, --verbose            be verbose\n"
-               "  -x, --export=<file>      export results in ORF format to <file>\n" );
-  if( verbosity > 0 )
+               "  -h, --help                display this help and exit\n"
+               "  -V, --version             output version information and exit\n"
+               "  -a, --append              append text to output file\n"
+               "  -c, --charset=<name>      try '--charset=help' for a list of names\n"
+               "  -e, --filter=<name>       try '--filter=help' for a list of names\n"
+               "  -E, --user-filter=<file>  user-defined filter, see manual for format\n"
+               "  -f, --force               force overwrite of output file\n"
+               "  -F, --format=<fmt>        output format (byte, utf8)\n"
+               "  -i, --invert              invert image levels (white on black)\n"
+               "  -l, --layout              perform layout analysis\n"
+               "  -o, --output=<file>       place the output into <file>\n"
+               "  -q, --quiet               suppress all messages\n"
+               "  -s, --scale=[-]<n>        scale input image by [1/]<n>\n"
+               "  -t, --transform=<name>    try '--transform=help' for a list of names\n"
+               "  -T, --threshold=<n%%>      threshold for binarization (0-100%%)\n"
+               "  -u, --cut=<l,t,w,h>       cut input image by given rectangle\n"
+               "  -v, --verbose             be verbose\n"
+               "  -x, --export=<file>       export results in ORF format to <file>\n" );
+  if( verbosity >= 1 )
     {
     std::printf( "  -1..6                    pnm output file type (debug)\n"
                  "  -C, --copy               'copy' input to output (debug)\n"
                  "  -D, --debug=<level>      (0-100) output intermediate data (debug)\n" );
     }
-  std::printf( "If no files are specified, ocrad reads the image from standard input.\n"
+  std::printf( "\nIf no files are specified, ocrad reads the image from standard input.\n"
                "If the -o option is not specified, ocrad sends text to standard output.\n"
                "\nExit status: 0 for a normal exit, 1 for environmental problems (file\n"
                "not found, invalid flags, I/O errors, etc), 2 to indicate a corrupt or\n"
@@ -166,7 +173,7 @@ void show_version()
   {
   std::printf( "GNU %s %s\n", program_name, PROGVERSION );
   std::printf( "Copyright (C) %s Antonio Diaz Diaz.\n", program_year );
-  std::printf( "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n"
+  std::printf( "License GPLv2+: GNU GPL version 2 or later <http://gnu.org/licenses/gpl.html>\n"
                "This is free software: you are free to change and redistribute it.\n"
                "There is NO WARRANTY, to the extent permitted by law.\n" );
   }
@@ -184,7 +191,7 @@ int process_file( FILE * const infile, const char * const infile_name,
                   const Input_control & input_control,
                   const Control & control )
   {
-  if( verbosity > 0 )
+  if( verbosity >= 1 )
     std::fprintf( stderr, "processing file '%s'\n", infile_name );
   try
     {
@@ -194,22 +201,22 @@ int process_file( FILE * const infile, const char * const infile_name,
       {
       if( page_image.cut( input_control.ltwh ) )
         {
-        if( verbosity > 0 )
+        if( verbosity >= 1 )
           std::fprintf( stderr, "file cut to %dw x %dh\n",
                         page_image.width(), page_image.height() );
         }
       else
         {
-        if( verbosity > 0 )
+        if( verbosity >= 1 )
           std::fprintf( stderr, "file '%s' totally cut away\n", infile_name );
         return 1;
         }
       }
 
     page_image.transform( input_control.transformation );
-    page_image.scale( input_control.scale );
+    page_image.change_scale( input_control.scale );
     page_image.threshold( input_control.threshold );
-    if( verbosity > 0 )
+    if( verbosity >= 1 )
       {
       const Rational th( page_image.threshold(), page_image.maxval() );
       std::fprintf( stderr, "maxval = %d, threshold = %d (%s)\n",
@@ -232,7 +239,7 @@ int process_file( FILE * const infile, const char * const infile_name,
       }
     }
   catch( Page_image::Error e ) { show_error( e.msg ); return 2; }
-  if( verbosity > 0 ) std::fprintf( stderr, "\n" );
+  if( verbosity >= 1 ) std::fputs( "\n", stderr );
   return 0;
   }
 
@@ -255,37 +262,39 @@ int main( const int argc, const char * const argv[] )
 
   const Arg_parser::Option options[] =
     {
-    { '1', 0,           Arg_parser::no  },
-    { '2', 0,           Arg_parser::no  },
-    { '3', 0,           Arg_parser::no  },
-    { '4', 0,           Arg_parser::no  },
-    { '5', 0,           Arg_parser::no  },
-    { '6', 0,           Arg_parser::no  },
-    { 'a', "append",    Arg_parser::no  },
-    { 'c', "charset",   Arg_parser::yes },
-    { 'C', "copy",      Arg_parser::no  },
-    { 'D', "debug",     Arg_parser::yes },
-    { 'e', "filter",    Arg_parser::yes },
-    { 'f', "force",     Arg_parser::no  },
-    { 'F', "format",    Arg_parser::yes },
-    { 'h', "help",      Arg_parser::no  },
-    { 'i', "invert",    Arg_parser::no  },
-    { 'l', "layout",    Arg_parser::no  },
-    { 'o', "output",    Arg_parser::yes },
-    { 'q', "quiet",     Arg_parser::no  },
-    { 's', "scale",     Arg_parser::yes },
-    { 't', "transform", Arg_parser::yes },
-    { 'T', "threshold", Arg_parser::yes },
-    { 'u', "cut",       Arg_parser::yes },
-    { 'v', "verbose",   Arg_parser::no  },
-    { 'V', "version",   Arg_parser::no  },
-    { 'x', "export",    Arg_parser::yes },
-    {  0 , 0,           Arg_parser::no  } };
+    { '1', 0,             Arg_parser::no  },
+    { '2', 0,             Arg_parser::no  },
+    { '3', 0,             Arg_parser::no  },
+    { '4', 0,             Arg_parser::no  },
+    { '5', 0,             Arg_parser::no  },
+    { '6', 0,             Arg_parser::no  },
+    { 'a', "append",      Arg_parser::no  },
+    { 'c', "charset",     Arg_parser::yes },
+    { 'C', "copy",        Arg_parser::no  },
+    { 'D', "debug",       Arg_parser::yes },
+    { 'e', "filter",      Arg_parser::yes },
+    { 'E', "user-filter", Arg_parser::yes },
+    { 'f', "force",       Arg_parser::no  },
+    { 'F', "format",      Arg_parser::yes },
+    { 'h', "help",        Arg_parser::no  },
+    { 'i', "invert",      Arg_parser::no  },
+    { 'l', "layout",      Arg_parser::no  },
+    { 'o', "output",      Arg_parser::yes },
+    { 'q', "quiet",       Arg_parser::no  },
+    { 's', "scale",       Arg_parser::yes },
+    { 't', "transform",   Arg_parser::yes },
+    { 'T', "threshold",   Arg_parser::yes },
+    { 'u', "cut",         Arg_parser::yes },
+    { 'v', "verbose",     Arg_parser::no  },
+    { 'V', "version",     Arg_parser::no  },
+    { 'x', "export",      Arg_parser::yes },
+    {  0 , 0,             Arg_parser::no  } };
 
   const Arg_parser parser( argc, argv, options );
   if( parser.error().size() )				// bad option
     { show_error( parser.error().c_str(), 0, true ); return 1; }
 
+  int retval = 0;
   int argind = 0;
   for( ; argind < parser.arguments(); ++argind )
     {
@@ -306,8 +315,10 @@ int main( const int argc, const char * const argv[] )
                 break;
       case 'C': input_control.copy = true; break;
       case 'D': control.debug_level = std::strtol( arg, 0, 0 ); break;
-      case 'e': if( !control.filter.set( arg ) )
-                  { control.filter.show_error( program_name, arg ); return 1; }
+      case 'e': if( !control.add_filter( program_name, arg ) ) return 1;
+                break;
+      case 'E': retval = control.add_user_filter( program_name, arg );
+                if( retval != 0 ) return retval;
                 break;
       case 'f': force = true; break;
       case 'F': if( !control.set_format( arg ) )
@@ -328,13 +339,13 @@ int main( const int argc, const char * const argv[] )
       case 'v': if( verbosity < 4 ) ++verbosity; break;
       case 'V': show_version(); return 0;
       case 'x': exportfile_name = arg; break;
-      default : Ocrad::internal_error( "uncaught option" );
+      default : Ocrad::internal_error( "uncaught option." );
       }
     } // end process options
 
 #if defined(__MSVCRT__) || defined(__OS2__) || defined(_MSC_VER)
-  setmode( STDIN_FILENO, O_BINARY );
-  setmode( STDOUT_FILENO, O_BINARY );
+  setmode( fileno( stdin ), O_BINARY );
+  setmode( fileno( stdout ), O_BINARY );
 #endif
 
   if( outfile_name && std::strcmp( outfile_name, "-" ) != 0 )
@@ -350,7 +361,7 @@ int main( const int argc, const char * const argv[] )
     if( !control.outfile )
       {
       if( verbosity >= 0 )
-        std::fprintf( stderr, "Cannot open %s\n", outfile_name );
+        std::fprintf( stderr, "Can't open '%s'\n", outfile_name );
       return 1;
       }
     }
@@ -365,7 +376,7 @@ int main( const int argc, const char * const argv[] )
       if( !control.exportfile )
         {
         if( verbosity >= 0 )
-          std::fprintf( stderr, "Cannot open %s\n", exportfile_name );
+          std::fprintf( stderr, "Can't open '%s'\n", exportfile_name );
         return 1;
         }
       }
@@ -377,7 +388,6 @@ int main( const int argc, const char * const argv[] )
   // process any remaining command line arguments (input files)
   FILE * infile = (argind < parser.arguments()) ? 0 : stdin;
   const char *infile_name = "-";
-  int retval = 0;
   while( true )
     {
     while( infile != stdin )
@@ -389,18 +399,18 @@ int main( const int argc, const char * const argv[] )
       else infile = std::fopen( infile_name, "rb" );
       if( infile ) break;
       if( verbosity >= 0 )
-        std::fprintf( stderr, "Cannot open %s\n", infile_name );
+        std::fprintf( stderr, "Can't open '%s'\n", infile_name );
       if( retval == 0 ) retval = 1;
       }
     if( !infile ) break;
 
-    int tmp = process_file( infile, infile_name, input_control, control );
+    const int tmp = process_file( infile, infile_name, input_control, control );
     if( infile == stdin )
       {
-      if( tmp <= 1 )
+      if( tmp <= 1 )		// detect EOF
         {
         int ch;
-        do ch = std::fgetc( infile ); while( std::isspace( ch ) );
+        do ch = std::fgetc( infile ); while( ch == 0 || std::isspace( ch ) );
         std::ungetc( ch, infile );
         }
       if( tmp > 1 || std::feof( infile ) || std::ferror( infile ) ) infile = 0;
