@@ -1,10 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
-    2012, 2013, 2014 Antonio Diaz Diaz.
+    Copyright (C) 2003-2015 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -144,7 +143,7 @@ Vrhomboid::Vrhomboid( const int l, const int lc, const int r, const int rc,
     if( verbosity >= 0 )
       std::fprintf( stderr, "l = %d, lc = %d, r = %d, rc = %d, h = %d\n",
                     l, lc, r, rc, h );
-    error( "bad parameter building a Vrhomboid" );
+    error( "bad parameter building a Vrhomboid." );
     }
   left_ = l; lvcenter_ = lc; right_ = r; rvcenter_ = rc; height_ = h;
   }
@@ -152,21 +151,21 @@ Vrhomboid::Vrhomboid( const int l, const int lc, const int r, const int rc,
 
 void Vrhomboid::left( const int l )
   {
-  if( l > right_ ) error( "left, bad parameter resizing a Vrhomboid" );
+  if( l > right_ ) error( "left, bad parameter resizing a Vrhomboid." );
   left_ = l;
   }
 
 
 void Vrhomboid::right( const int r )
   {
-  if( r < left_ ) error( "right, bad parameter resizing a Vrhomboid" );
+  if( r < left_ ) error( "right, bad parameter resizing a Vrhomboid." );
   right_ = r;
   }
 
 
 void Vrhomboid::height( const int h )
   {
-  if( h <= 0 ) error( "height, bad parameter resizing a Vrhomboid" );
+  if( h <= 0 ) error( "height, bad parameter resizing a Vrhomboid." );
   height_ = h;
   }
 
@@ -174,7 +173,7 @@ void Vrhomboid::height( const int h )
 void Vrhomboid::extend_left( const int l )
   {
   if( l > right_ )
-    error( "extend_left, bad parameter resizing a Vrhomboid" );
+    error( "extend_left, bad parameter resizing a Vrhomboid." );
   lvcenter_ = vcenter( l ); left_ = l;
   }
 
@@ -182,7 +181,7 @@ void Vrhomboid::extend_left( const int l )
 void Vrhomboid::extend_right( const int r )
   {
   if( r < left_ )
-    error( "extend_right, bad parameter resizing a Vrhomboid" );
+    error( "extend_right, bad parameter resizing a Vrhomboid." );
   rvcenter_ = vcenter( r ); right_ = r;
   }
 
@@ -294,7 +293,7 @@ int Track::vcenter( const int col ) const
   return 0;
   }
 
-
+/*
 bool Track::includes( const Rectangle & r ) const
   {
   for( unsigned i = 0; i < data.size(); ++i )
@@ -334,4 +333,4 @@ bool Track::includes( const int row, const int col ) const
     return tmp.includes( row, col );
     }
   return false;
-  }
+  }*/
